from typing import Any
from cmq.aws.aws import AWSResource


class sns(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "sns"
        self._resource = "sns"
        self._list_function = "list_topics"

        self._tag_function = "list_tags_for_resource"
        self._tag_function_key = "ResourceArn"

        self._metric_namespace = "AWS/SNS"
        self._metric_dimension_name = "TopicName"
        self._metric_dimension_resource_key = "TopicName"

    def _get_tag_resource_identifier(self, context: dict[str, Any], resource: dict[str, Any]) -> str:
        return resource["TopicArn"]



from cmq.aws.aws import AWSResource


class s3_object(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "s3"
        self._resource = "object"
        self._list_function = "list_objects_v2"

    def get_parameters(self, context: dict) -> dict:
        parameters = self._list_parameters.copy()
        if 's3_bucket' in context:
            parameters["Bucket"] = context["s3_bucket"]["Name"]
        return parameters


from cmq.aws.aws import AWSResource


class address(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "ec2"
        self._resource = "address"
        self._list_function = "describe_addresses"
        self._list_key = "Addresses"
        self._list_paginated = False



from cmq.aws.aws import AWSResource


class log_event(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "logs"
        self._resource = "log-event"
        self._list_function = "filter_log_events"
        self._list_paginated = False

    def get_parameters(self, context: dict) -> dict:
        parameters = self._list_parameters.copy()
        if 'logs_log-group' in context:
            parameters["logGroupName"] = context["logs_log-group"]["logGroupName"]
        if 'logs_log-stream' in context:
            parameters["logStreamNames"] = [context["logs_log-stream"]["logStreamName"]]
        return parameters



from cmq.aws.aws import AWSResource


class ec2(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "ec2"
        self._resource = "instance"
        self._list_function = "describe_instances"

    def _get_results(self, page) -> list | None:
        instances = []
        reservations = page.get("Reservations", [])
        for reservation in reservations:
            for instance in reservation.get("Instances", []):
                instances.append(instance)
        return instances



from typing import Any
from cmq.aws.aws import AWSResource


class dynamodb(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "dynamodb"
        self._resource = "dynamodb"
        self._list_function = "list_tables"

        self._describe_function = "describe_table"
        self._describe_function_key = "TableName"
        self._describe_resource_key = "resource"

        self._tag_function = "list_tags_of_resource"
        self._tag_function_key = "ResourceArn"
        self._tag_resource_key = "resource"

        self._metric_namespace = "AWS/DynamoDB"
        self._metric_dimension_name = "TableName"
        self._metric_dimension_resource_key = "resource"

    def _get_tag_resource_identifier(self, context: dict[str, Any], resource) -> str:
        return f"arn:aws:dynamodb:{context['aws_region']}:{context['aws_account']}:table/{resource['resource']}"
